/**
 * Declare the global variables that will be exported by this script.
 */
var $package;
var $identify;
var $require;
var $contains;
var $main;
var $extend;
var $baseurl;
var $setOptions;

/**
 * Enter a private variable context, so that variables declared below are not
 * visible outside this script.
 */
(function(){

  /**
   * Contains a record of all $main arguments that have been executed. This is
   * used by the $main and $require functions to ensure that each $main argument
   * is executed only once.
   */
  var executed = {
    urls: {},
    nourl: {}
  };

  /**
   * This counter is used to keep track of how many urls were requested to be
   * loaded (using the $require function), but haven't finished loading yet.
   */
  var loading = 0;

  /**
   * A list of functions that wish to be called when the 'loading' counter
   * reaches 0.
   */
  var waiting = { 
    urls:{},
    nourl:[] 
  };

  /**
   * A list of scripts already added
   */
  var loadingurls = {};

  /**
   * Holds a reference to the script tag that was last added to the document by
   * the $require function. New scripts will be added below this script.
   */
  var lastadded;

  /**
   * Record the global variable scope in the 'global' object. This is used by
   * the $package function to create new package objects in the global scope.
   */
  var global = this;

  /**
   * Returns the package object for the specified package string. For instance,
   * the call $package("org.mathdox.javascript") will return the
   * org.mathdox.javascript object. If the package object doesn't exist, it is
   * created.
   */
  $package = function(string) {

    var parts = string.split(".");
    var parent = global;
    var i;

    for (i=0; i<parts.length; i++) {
      if (!parent[parts[i]]) {
        parent[parts[i]] = {};
      }
      parent = parent[parts[i]];
    }
    return parent;

  };

  /**
   * url of the currently procressed file 
   */
  var current={}; 

  /**
   * set the url of the currently processed file
   */
  $identify = function(name) {
    current.url = name;
  };

  var execute_with_requirements = function(url) {
    if (executed.urls[url]) {
      return;
    }

    if (!waiting.urls[url]) {
      return;
    }
	
    var requirements = waiting.urls[url].requirements;
    if (requirements && requirements.length) {
      var i;
      for (i=requirements.length-1; i>=0; i--) {
	execute_with_requirements(requirements[i].url);
      }
    }
    /*alert("executing: "+url);*/
    waiting.urls[url].continuation();
    executed.urls[url] = true;
  };

  /**
   * When a script has been loaded, this function will check whether all
   * requirements have been loaded, and will then execute the main sections.
   */
  var onload = function() {

    loading = loading - 1;
    if (loading === 0) {
      while (waiting.nourl.length > 0) {
	var nourl = waiting.nourl.pop();
	var urlobj;
	for (urlobj in nourl.requirements) {
	  execute_with_requirements(nourl.requirements[urlobj].url);
	}
        var continuation = nourl.continuation;
        if (!executed.nourl[continuation]) {
          executed.nourl[continuation] = true;
          continuation();
        }
      }

      var url;
      for (url in waiting.urls) {
	execute_with_requirements(url);
      }
    }

  };

  /**
   * Ensures that the script at the specified url is loaded before the main
   * section is executed.
   * We cannot detect whether a script has finished loading when a third-party
   * script is loaded that doesn't contain a call to $main. In those cases a
   * 'ready' function to detect load completion must be specified. This function
   * should return 'true' when the script has loaded, and 'false' otherwise.
   * Typically, such a function would check for the presence of a variable that
   * is set by the third-party script.
   */

  /* delayed require */
  $require = function(url, ready) {
    if (!current.requirements) {
      current.requirements = [];
    }
    current.requirements.push( { url:url, ready:ready } );
  };

  var require_action = function(url, ready) {
    /* already being loaded*/
    if (waiting.urls[url] || loadingurls[url]) {
      return;
    }

    /* store that this url has been added*/
    loadingurls[url] = true;

    /* add base url to url, unless it is an absolute url*/
    if (!url.match(/^\/|:/)) {
      url = $baseurl + url;
    }

    /* increase the 'loading' counter*/
    loading = loading + 1;

    /* add script tag to document*/
    var script = document.createElement("script");
    script.type = "text/javascript";
    script.src = url;
    lastadded.parentNode.insertBefore(script, lastadded.nextSibling);
    lastadded = script;

    /* use the 'ready' function to check for load complete, if it was specified */
    if (ready instanceof Function) {
      var wait = function() {
        if (ready()) {
          onload();
        }
        else {
          setTimeout(wait, 100);
        }
      };
      wait();
    }

  };

  /**
   * Calls the continuation function that is specified as a parameter when all
   * previously specified requirements have been loaded. Each main section is
   * only executed once.
   */
  $main = function(continuation) {
    current.continuation = continuation;

    /* add current to waiting queue */
    if (current.url) {
      waiting.urls[current.url] = current;
    } else {
      waiting.nourl.push({
	continuation:continuation, 
	requirements:current.requirements
      });
    }

    /* store requirements and reset current */
    var requirements = current.requirements;

    current={};

    /* do delayed requires and create waiting queue */
    if (requirements) {
      var i;

      for (i = requirements.length - 1; i>=0; i--) {
	var req = requirements[i];

	require_action(req.url, req.ready);
      }
    }
    /* call onload function */
    onload();
  };

  /**
   * Returns a new class that adds the specified properties to the specified
   * parent class.
   */
  $extend = function(parent, properties) {

    /* figure out what the prototype of the new class will be */
    var prototype;
    var x;

    if (parent instanceof Function) {
      prototype = new parent();
    }
    else {
      var parentConstructor = function(){};
      parentConstructor.prototype = parent;
      prototype = new parentConstructor();
    }

    /* create the new class constructor */
    var constructor = function(){
      if (this.initialize instanceof Function) {
        this.initialize.apply(this, arguments);
      }
    };

    /* copy any static class properties of the parent to the new class */
    if (parent instanceof Function) {
      for (x in parent) {
        constructor[x] = parent[x];
      }
    }

    /* copy the additional properties to the prototype */
    for (x in properties) {

      /* add a 'parent' property to methods, to enable super calls */
      if (properties[x] instanceof Function) {
        if (!("parent" in properties[x])) {
          if (parent instanceof Function) {
            properties[x].parent = parent.prototype;
          }
          else {
            properties[x].parent = parent;
          }
        }
      }

      prototype[x] = properties[x];

    }

    /* combine constructor and prototype */
    constructor.prototype = prototype;

    /* return new class constructor */
    return constructor;

  };

  /*
   * Help function to indicate that more files are concatenated and do not
   * needed to be loaded seperately.
   */
  $contains = function(url) {
    loadingurls[url] = true;
    loading += 1;
  };

  $setOptions = function(prefix, optionList) {
    var options = $package(prefix);
    var optionName;
    for (optionName in optionList) {
      options[optionName] = optionList[optionName];
    }
    return options;
  };

  /**
   * Find the script tag that was used to load this script, and use it to
   * calculate the value of 'lastadded' and '$baseurl'.
   */
  var scripts = document.getElementsByTagName("script");
  var scriptfinder1 = /(.*)wrapper\.js$/;
  var scriptfinder2 = /(.*)functions_pm\.js$/;

  var i;
  var match;
  for (i=0; i<scripts.length; i++) {
    match = scripts[i].src.match(scriptfinder1);
    if (match === null) {
      match = scripts[i].src.match(scriptfinder2);
    }
    if (match !== null) {
      lastadded = scripts[i];
      $baseurl = match[1];
    }
  }

})();
/* version: 0.10.7.3 */
$contains("org/mathdox/ButtonPusher.js");
$contains("org/mathdox/Debug.js");
$contains("org/mathdox/DebugClient.js");
$contains("org/mathdox/EditorInteraction.js");
$contains("org/mathdox/EventLib.js");
$contains("org/mathdox/ExerciseInteraction.js");
$contains("org/mathdox/FakeAPI.js");
$contains("org/mathdox/InitLMS.js");
$contains("org/mathdox/MessageEventLib.js");
$contains("org/mathdox/PostDebug.js");
$contains("org/mathdox/PostMessage.js");
$contains("org/mathdox/SCORM.js");
$contains("org/mathdox/SCORMServer.js");
$contains("org/mathdox/ScoreListener.js");
$contains("org/mathdox/Session.js");
$contains("org/mathdox/XML.js");
$package("org.mathdox");

$identify("org/mathdox/ButtonPusher.js");

$require("org/mathdox/PostMessage.js");

$main(function() {
  org.mathdox.ButtonPusher = $extend(Object, {

    interaction : null,

    iframe : null,
    mode: null,

    idstr : "MathDox Exercise System - Button Pusher",

    initialize : function(mode, iframe) {
      if (mode == "scorm") {
	this.mode = mode;
      } else if (mode == "wrapper" && iframe !== undefined && iframe !== null) {
	this.iframe = iframe;
	this.mode = mode;
      } else {
	console.log("Warning: could not initialize ButtonPusher");
      }
    },

    addMessageListenersScorm: function(messageEventLib) {
      // no listener in scorm
    },
     
    addMessageListenersWrapper: function(messageEventLib) {
      this.messageListener.buttonPusher = this;
      messageEventLib.addMessageListener(this.idstr, "button", this.messageListener);
    },

    pushButton: function(name) {
      if (this.iframe !== null && this.iframe !== undefined) {
        this.pushButtonInIFrame(name);
      } else {
        this.sendPushButton(name);
      }
    },

    pushButtonInIFrame: function(name) {
      var buttons = this.iframe.contentDocument.getElementsByTagName("button");
      var i;

      if (name === undefined) {
	return;
      }

      for (i=0; i<buttons.length; i++) {
	if (buttons[i].textContent == name || buttons[i].innerText == name) {
	  buttons[i].click();
	}
      }
    },
 
    messageListener: function(event) {
      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);
      var buttonPusher = arguments.callee.buttonPusher;
      
      if (data !== undefined && data !== null) {
        if (data.idstr == buttonPusher.idstr) {
	  if (data.mode == "button") {
	    buttonPusher.pushButtonInIFrame(data.name);
	  }
	}
      }
    },

    sendPushButton: function(name) {
      var pm = new org.mathdox.PostMessage();

      var data = {
        idstr : this.idstr,
        mode : "button",
        name : name
      };

      pm.sendMessageIFrames(data);
    }
  });

});
$package("org.mathdox");

$identify("org/mathdox/Debug.js");

$require("org/mathdox/PostMessage.js");

$main(function() {
  org.mathdox.Debug = $extend(Object, {

    //other variables
    debug             : false,
    debugBuffer       : "",
    debugDiv          : null,
    debugLevel        : 5,
    frameInitTimeStep : 10,
    idstr             : "MathDox Exercise System - PostDebug",
    
    initialize: function(debug) {
      this.debugChildInit.idstr = this.idstr;
      if (debug !== undefined && debug!==null) {
        this.debug = debug;
      }
    },

    addDebug: function(text, level) {
      if (level === undefined || level === null) {
        level = 0;
      }

      if(this.debug && level <= this.debugLevel) {
        try {
          this.debugDiv.innerHTML+=text+'<br>';
        } catch(err) {
          this.debugBuffer+=text+'<br>';
        }
      }
    },

    addMessageListeners: function(messageEventLib) {
      this.messageListener.debug = this;
      messageEventLib.addMessageListener(this.idstr, "debug", this.messageListener);
    },

    createDebug: function() {
      var debugDiv;
      var obj;
      if(this.debug) {
        debugDiv = document.createElement("div");
        debugDiv.id="debugWindow";
        debugDiv.style.position="absolute";
        debugDiv.style.right="5px";
        debugDiv.style.top="5px";
        debugDiv.style.borderStyle="solid";
        debugDiv.style.borderColor="#000000";
        debugDiv.style.borderWidth="1px";
        debugDiv.style.backgroundColor="#EEEEEE";
        debugDiv.style.padding="5px";
        debugDiv.style.fontSize="12px";
        debugDiv.style.zIndex="-1";
        debugDiv.innerHTML="<b>Debug-window</b><br />"+this.debugBuffer;
        this.debugDiv = debugDiv;
        try {
          this.addDebug("- add debug window");
          document.body.insertBefore(debugDiv,document.body.firstChild);
        } catch(err) {
          this.addDebug("- delayed adding debug window (not ready)");
          obj = this;

          setTimeout(function() {obj.createDebug(); }, this.frameInitTimestep);
        }
      }
    },
  
    debugChildInit: function() {
      var data = {
        idstr : arguments.callee.idstr,
        mode  : "debugInit"
      };
      var pm = new org.mathdox.PostMessage();
      var list = document.getElementsByTagName('iframe');

      var i;
      var iframe;
      for(i=0;i<list.length;i++) {
        iframe=list[i];

	pm.sendMessage(iframe.contentWindow, data, "*");
      }
    },

    messageListener: function(event) {
      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);

      if (data !== undefined && data !== null) {
        if (data.idstr == arguments.callee.debug.idstr) {
          if (data.mode == "debug") {
            if (data.text !== undefined && data.text !== null) {
              arguments.callee.debug.addDebug("[child] "+data.text);
            }
          }
        }
      }
    },

    startDebug: function() {
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/DebugClient.js");

$main(function() {
  org.mathdox.DebugClient = $extend(Object, {
    debug: null,

    initialize: function(debug) {
      if (debug !== undefined && debug !== null) {
        this.debug = debug;
      }
    },
    
    addDebug: function(text, level) {
      if (this.debug !== null) {
        this.debug.addDebug(text, level);
      }
    }
  });
});

$package("org.mathdox");

$identify("org/mathdox/EditorInteraction.js");

$require("org/mathdox/PostMessage.js");

$main(function() {
  org.mathdox.EditorInteraction = $extend(Object, {

    iframe : null,
    mode: null,
    debug: null,

    idstr : "MathDox Exercise System - Editor Interaction",

    initialize : function(mode, iframe, debug) {
      if (mode == "scorm") {
	this.mode = mode;
      } else if (mode == "wrapper" && iframe !== undefined && iframe !== null) {
	this.iframe = iframe;
	this.mode = mode;
      } else {
	console.log("Warning could not initialize EditorInteraction");
      }

      if (debug !== undefined && debug !== null) {
	this.debug = debug;
      }
    },

    addDebug: function(msg) {
      if (this.debug !== null && this.debug !== undefined) {
        this.debug.addDebug(msg);
      }
    },

    addMessageListenersScorm: function(messageEventLib) {
      // no listener in scorm
    },
     
    addMessageListenersWrapper: function(messageEventLib) {
      this.messageListener.editorInteraction = this;
      //messageEventLib.addMessageListener(this.idstr, "loadMathML", this.messageListener);
      messageEventLib.addMessageListener(this.idstr, "updateOpenMath", this.messageListener);
      //messageEventLib.addMessageListener(this.idstr, "appendText", this.messageListener);
    },

    updateOpenMath: function(openmath, index) {
      if (index === undefined  || index === null) {
	index = 0;
      }
      if (this.iframe !== null && this.iframe !== undefined) {
        this.updateOpenMathInIFrame(openmath, index);
      } else {
        this.sendUpdateOpenMath(openmath, index);
      }
    },

    updateOpenMathInIFrame: function(openmath, index) {
      this.addDebug("EditorInteraction: updating openmath in iframe");

      var editorTags = this.iframe.contentDocument.getElementsByClassName("mathdoxformula");
      var editors = [];

      var i;

      this.addDebug("EditorInteraction: checking elements with class mathdoxformula ("+editorTags.length+")");

      for (i=0; i<editorTags.length; i++) {
        if (editorTags[i].tagName == "TEXTAREA") {
          editors.push(this.iframe.contentWindow.org.mathdox.formulaeditor.FormulaEditor.getEditorByTextArea(editorTags[i]));
        }
      }
      this.addDebug("EditorInteraction: editors found: " + editors.length);
      
      if (editors[index] !== undefined && editors[index] !== null) {
        this.addDebug("EditorInteraction: editor "+index+" found");
        
	this.addDebug("EditorInteraction: editor "+index+" found");
	editors[index].textarea.value = openmath;
        editors[index].load();

	// necessary for Orbeon
        editors[index].save();

        this.addDebug("EditorInteraction: updating orbeon");
        if (this.iframe.contentWindow.ORBEON && editors[index].textarea.id) {
          this.iframe.contentWindow.ORBEON.xforms.Document.setValue(editors[index].textarea.id, openmath);
          this.addDebug("EditorInteraction: orbeon updated");
        } else {
          this.addDebug("EditorInteraction: Warning: could not store in ORBEON");
	}

        editors[index].redraw();
      } else {
        this.addDebug("EditorInteraction: invalid editor index "+index);
      }
    },
 
    messageListener: function(event) {
      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);
      var editorInteraction = arguments.callee.editorInteraction;
      
      if (data !== undefined && data !== null) {
        if (data.idstr == editorInteraction.idstr) {
	  if (data.mode == "updateOpenMath") {
	    editorInteraction.updateOpenMathInIFrame(data.openmath, data.index);
	  }
	}
      }
    },

    sendUpdateOpenMath: function(openmath, index) {
      this.addDebug("EditorInteraction: requesting updateOpenMath with postmessage (index "+index+")");
      var pm = new org.mathdox.PostMessage();

      var data = {
        idstr : this.idstr,
        index: index,
        mode : "updateOpenMath",
        openmath : openmath
      };

      pm.sendMessageIFrames(data);
    }
  });

});
$package("org.mathdox");

$identify("org/mathdox/EventLib.js");

$main(function () {
  org.mathdox.EventLib = $extend(Object, {
    addEventListener: function(type, listener, useCapture) {
      if (window.addEventListener) {
        if ((type === "load") && (document.readyState && document.readyState == "complete")) {
          /* in Chrome the page might be loaded already */
          listener();
        } else {
          window.addEventListener(type, listener, useCapture);
        }
      } /* NOTE: IE before 9 code with attachEvent removed */
    },

    /* -------------------------------------------------------------------
       Event handler (for FF and IE)
       -------------------------------------------------------------------
    */
  
    registerEventHandler: function(node, event, handler) {
      if (typeof node.addEventListener == "function") {
        node.addEventListener(event, handler, false);
      } else {
        node.attachEvent("on" + event, handler);
      }
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/ExerciseInteraction.js");

$require("org/mathdox/PostMessage.js");

$main(function() {
  org.mathdox.ExerciseInteraction = $extend(Object, {

    debug : null,
    interaction : null,
    iframe : null,

    loadFunctions : [],
    updateFunctions : [],
    variables: [],

    idstr : "MathDox Exercise System - Exercise Interaction",

    initialize: function(debug, iframe) {
      this.sendInteraction.obj = this;

      if (debug !== null && debug !== undefined) {
	this.debug = debug;
      }
      
      this.iframe = iframe;
    },
    
    addDebug: function(msg) {
      if (this.debug !== null && this.debug !== undefined) {
	this.debug.addDebug(msg);
      }
    },

    addMessageListenersScorm: function(messageEventLib) {
      this.messageListener.exerciseInteraction = this;
      messageEventLib.addMessageListener(this.idstr, "interaction", this.messageListener);
    },
     
    addMessageListenersWrapper: function(messageEventLib) {
    },

    addLoadListener: function(fun) {
      this.loadFunctions.push(fun);
    },

    callLoadListeners: function() {
      var i;

      this.addDebug("Calling load Listeners for interaction "+this.interaction);
      for (i=0;i<this.loadFunctions.length;i++) {
        (this.loadFunctions[i])(this.interaction, this.variables);
      }
    },

    addUpdateListener: function(fun) {
      this.updateFunctions.push(fun);
    },
   
    callInteractionUpdateListeners: function() {
      var i;

      this.debug.addDebug("Calling update Listeners for interaction "+this.interaction);
      for (i=0;i<this.updateFunctions.length;i++) {
        (this.updateFunctions[i])(this.interaction, this.variables);
      }
    },
   
    getInteractionFromIFrame: function(iframe) {
      if (iframe.contentDocument === null || iframe.contentDocument === undefined) {
        return null;
      }

      var interactions = iframe.contentDocument.getElementsByClassName("mathdox-interaction");
      if (interactions.length >=1) {
	var content = interactions[0].textContent;
	if (content !== undefined) {
	  return content;
	} 
        return interactions[0].innerText;
      } else {
        return null;
      }
    },

    getVariablesFromCache: function() {
      return this.variables;
    },

    getVariablesFromIFrame: function(iframe) {
      if (iframe.contentWindow === null || iframe.contentWindow === undefined) {
        return null;
      }

      var variables = iframe.contentWindow.Mathdox.LMS.data.shared_variables; 

      return variables;
    },

    messageListener: function(event) {
      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);
      var exerciseInteraction = arguments.callee.exerciseInteraction;
      
      if (data !== undefined && data !== null) {
        if (data.idstr == exerciseInteraction.idstr) {
	  if (data.mode == "interaction") {
	    exerciseInteraction.interaction = data.interaction;
            if (data.variables !== undefined && data.variables !== null && data.variables.trim() !== "") {
              try {
                var variables = JSON.parse(data.variables);
		
		var variables_decoded= {};
		for (name in variables) {
                  variables_decoded[name] = variables[name].replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&quot;/g, "\"").replace(/&#x27;/g, "'");
		}

	        exerciseInteraction.variables = variables_decoded;
              } catch (e) {
                console.log("Error while parsing variables '"+data.variables+"' :"+e);
              }
            }
            exerciseInteraction.callInteractionUpdateListeners();
	  }
	}
      }
    },

    sendInteraction: function(iframe) {
      var obj = arguments.callee.obj;

      var interaction = obj.getInteractionFromIFrame(iframe);
      var variables = obj.getVariablesFromIFrame(iframe);

      obj.interaction = interaction;
      
      obj.addDebug("sending Interaction "+interaction);

      var data = {
        idstr: obj.idstr,
        mode: "interaction",
        interaction: interaction,
        variables: variables
      }

      var pm = new org.mathdox.PostMessage(obj.debug);

      pm.sendMessageParent(data, '*');
    }
  });

});
$package("org.mathdox");

$identify("org/mathdox/Fakeapi.js");

$main(function() {
  org.mathdox.FakeAPI = $extend(Object, {
    debug: null,

    data : {
      "cmi.core.student_id": "test",
      "cmi.core.student_name": "Test Student (Fake API)",
      "cmi.core.credit": "credit",
      "cmi.core.lesson_status": "incomplete",
      "cmi.core.entry": "",
      "cmi.launch_data": "&lt;package_info&gt; &lt;package&gt;Test Package&lt;/package&gt; &lt;package_id&gt;4763&lt;/package_id&gt; &lt;author&gt;Jan Willem Knopper&lt;/author&gt; &lt;author_id&gt;12&lt;/author_id&gt; &lt;exercise&gt;1&lt;/exercise&gt; &lt;exercise_id&gt;100000&lt;/exercise_id&gt; &lt;course&gt;Test course &lt;/course&gt; &lt;course_id&gt;12&lt;/course_id&gt; &lt;organization&gt;Test Section&lt;/organization&gt; &lt;organization_id&gt;0&lt;/organization_id&gt; &lt;organization_content&gt;1&lt;/organization_content&gt; &lt;organization_content_id&gt;2347&lt;/organization_content_id&gt; &lt;spg&gt; &lt;hostname&gt;evo01.win.tue.nl&lt;/hostname&gt; &lt;/spg&gt; &lt;created&gt;1383570673&lt;/created&gt; &lt;url&gt;http://evo01.win.tue.nl/mathadore/mathdox/wrapper/test/exercise1.xml&lt;/url&gt; &lt;/package_info&gt;",
      "cmi.student_data.mastery_score": "69"
    },

    initialize: function(debug) {
      if (debug !== undefined && debug !== null) {
        this.debug = debug;
      }
    },

    addDebug: function(text, level) {
      if (this.debug !== null) {
        this.debug.addDebug(text, level);
      }
    },

    // From SCORM.js
    getData: function(key) {
      if (key === null || key === undefined ||key === "undefined") {
        return null;
      }
      if (this.data[key]!== undefined) {
        return this.data[key];
      }
    },
    
    LMSCommit: function(str) {
      this.addDebug("Fake SCORM api: LMScommit called");
      return 'true';
    },

    LMSFinish: function(str) {
      this.addDebug("Fake SCORM api: LMS session finished");
      return 'true';
    },

    LMSGetLastError: function() {
      return 'unknown error';
    },

    LMSGetValue: function(key) {
      if (key == "cmi.launch_data") {
        this.addDebug("Fake SCORM api: LMSGetValue "+key);
      } else {
        this.addDebug("Fake SCORM api: LMSGetValue "+key+": "+this.getData(key));
      }

      return this.getData(key);
    },

    LMSInitialize: function(str) {
      this.addDebug("Fake SCORM api: LMS session Initialized");
      return 'true';
    },

    LMSSetValue: function(key, value) {
      this.addDebug("Fake SCORM api: LMSSetValue "+key+": "+this.getData(key)+"->"+value);
      this.setData(key, value);
    },

    // from SCORM.js
    setData: function(key, value) {
      if (key !== null && key!== undefined) {
        this.data[key] = value;
      }
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/InitLMS.js");

$require("org/mathdox/Debug.js");
$require("org/mathdox/ButtonPusher.js");
$require("org/mathdox/EditorInteraction.js");
$require("org/mathdox/ExerciseInteraction.js");
$require("org/mathdox/FakeAPI.js");
$require("org/mathdox/MessageEventLib.js");
$require("org/mathdox/SCORMServer.js");

var Mathdox ;

$main(function() {
  var debug = new org.mathdox.Debug(true);
  var scorm = new org.mathdox.SCORMServer(debug);
  var exerciseInteraction = new org.mathdox.ExerciseInteraction(debug);
  var messageEventLib = new org.mathdox.MessageEventLib();
  
  debug.createDebug();
  debug.addMessageListeners(messageEventLib);
  scorm.addMessageListeners(messageEventLib);
  exerciseInteraction.addMessageListenersScorm(messageEventLib);

  var interactionDebug = function(interaction) {
    debug.addDebug("Interaction: "+interaction);
  };

  exerciseInteraction.addUpdateListener(interactionDebug);

  if (Mathdox === undefined || Mathdox === null) {
    Mathdox = {};
  }

  Mathdox.addInteractionListener = function(fun) {
    exerciseInteraction.addUpdateListener(fun);
  }
  
  Mathdox.requestVariable = function(name, callback) {
    if (callback !== undefined && callback !== null) {
      exerciseInteraction.addVariableCallbackFunction(name, callback);
    }
    exerciseInteraction.requestVariable(name);
  }
  
  var editorInteraction = new org.mathdox.EditorInteraction("scorm", null, debug);
  Mathdox.updateOpenMath = function(openmath, index) {
    debug.addDebug("updating openmath for index "+index);
    editorInteraction.updateOpenMath(openmath, index);
  }

  var buttonPusher = new org.mathdox.ButtonPusher("scorm");
  Mathdox.pushButton = function(name) {
    buttonPusher.pushButton(name);
  }

  messageEventLib.addMessageListeners();
  debug.addDebug("done with initialization");

  var initfun = function() {
    var list = document.getElementsByTagName('iframe');
    var initfuniframe;
    var iframe;
    var i;
    for(i=0;i<list.length;i++) {
      iframe=list[i];      

      initfuniframe = function() {
        var iframe = arguments.callee.iframe;
        if (iframe.contentWindow !== null) {
          debug.addDebug("- calling lmsInit");
          scorm.sendLMSInit.call(scorm, iframe);
        } else {
          debug.addDebug("- delayed calling lmsInit");
          setTimeout(arguments.callee, 100);
        }
      };

      initfuniframe.iframe = iframe;

      initfuniframe();
    }
  };

  var commitfun = function() {
    scorm.apiLMSFinish();
  };

  messageEventLib.addEventListener("load", initfun, false);
  //messageEventLib.addEventListener("unload", commitfun, false);
});
$package("org.mathdox");

$identify("org/mathdox/MessageEventLib.js");

$require("org/mathdox/EventLib.js");
$require("org/mathdox/PostMessage.js");

$main(function () {
  org.mathdox.MessageEventLib = $extend(org.mathdox.EventLib, {
    idstrList : {},

    /* note: currently only one function supported per idstr/mode */
    addMessageListener: function(idstr, mode, fun) {
      if (idstr === undefined || idstr === null || mode === undefined || mode === null) {
        return;
      }
      if (this.idstrList[idstr] === undefined || this.idstrList[idstr] === null) {
        this.idstrList[idstr]={};
      }
      if (this.idstrList[idstr][mode] === undefined || this.idstrList[idstr][mode] === null) {
        this.idstrList[idstr][mode]=fun;
      }
    },

    addMessageListeners: function(messageEventLib) {
      this.messageListener.obj = this;
      this.addEventListener("message", this.messageListener, false);
    },


    messageListener: function(event) {
      var data;
      var idstrList;
      var pm = new org.mathdox.PostMessage();

      var obj = arguments.callee.obj;

      data = pm.decodeMessage(event.data);
      
      if (data !== undefined && data !== null) {
        idstrList = arguments.callee.obj.idstrList;

        if (data.idstr !== undefined && data.idstr !== null && idstrList[data.idstr] !== undefined && idstrList[data.idstr] !== null) {
          if (data.mode !== undefined && data.mode !== null && idstrList[data.idstr][data.mode] !== undefined && idstrList[data.idstr][data.mode] !== null) {
            idstrList[data.idstr][data.mode](event);
          }
        }
      }
    },

    sendMessageIFrames: function(data, host) {
      var pm = new org.mathdox.PostMessage();
      
      pm.sendMessageIFrames(data, host);
    },

    sendMessageParent: function(data, host) {
      var pm = new org.mathdox.PostMessage();

      pm.sendMessageParent(data, host);
    }

  });
});
$package("org.mathdox");

$identify("org/mathdox/PostDebug.js");

$require("org/mathdox/Debug.js");
$require("org/mathdox/PostMessage.js");

$main(function() { 
  org.mathdox.PostDebug = $extend(org.mathdox.Debug, {
    started: false,
    debug: true,

    addDebug: function(text) {
      if(this.debug) {
	if (this.started !== false) {
          var data = {
            idstr : this.idstr,
            mode  : "debug",
            text  : text
          };

	  var pm = new org.mathdox.PostMessage();
	  pm.sendMessageParent(data, '*');
        } else {
          this.debugBuffer+=text+'<br>';
        }
      }
    },

    addMessageListeners: function(messageEventLib) {
      this.messageListenerInit.debug = this;
      messageEventLib.addMessageListener(this.idstr, "debugInit", this.messageListenerInit);

      var parent = arguments.callee.parent;
      parent.addMessageListeners.call(this, messageEventLib);
    },

    createDebug: function() {
      this.addDebug("- created PostDebug");
      /* do nothing */
    },
 
    messageListenerInit: function(event) {
      var debug = arguments.callee.debug;

      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);

      if (data !== undefined && data !== null) {
        if (data.idstr == debug.idstr) {
          if (data.mode == "debugInit") {
            debug.startDebug();
          }
        }
      }
    },

    startDebug: function() {
      if (this.started === false) {
        this.started = true;
        if (this.debugBuffer != "") {
          var buffer = this.debugBuffer;
          this.debugBuffer = "";
          this.addDebug(buffer);
        }
      }
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/PostMessage.js");

$main(function() {
  org.mathdox.PostMessage = $extend(Object, {
    debug: null,

    initialize: function(debug) {
      if (debug !== undefined && debug !== null) {
        this.debug = debug;
      }
    },

    addDebug: function(debugStr) {
      var pos;

      if ( this.debug !== null && this.debug !== undefined) {
        for (pos = 0; pos < debugStr.length; pos += 40) {
	  if (pos + 40 <debugStr.length) {
            this.debug.addDebug(debugStr.substring(pos, pos+40));
	  } else {
            this.debug.addDebug(debugStr.substring(pos));
	  }
	}
      }
    },

    decodeMessage: function(encdata) {
      var data;

      if (typeof encdata === "object" && encdata.constructor !== String) {
        data = encdata;
      } else if (typeof encdata == "string" || (typeof encdata == "object" && encdata.constructor === String)) { 
        /* String, try to decode as JSON */
	try {
          data = JSON.parse(encdata);
	} catch (e) {
          data = null;
	}
      }

      if (data === "undefined") {
	return null;
      }

      return data;
    },
    
    encodeMessage: function(data) {
      return JSON.stringify(data);
    },

    sendMessage: function(target, data, host) {
      var encdata = this.encodeMessage(data);

      if (host === null || host === undefined) {
        host = "*";
      }

      if (target !== undefined && target !== null) {
        this.addDebug("sending message: " + encdata);
        target.postMessage(encdata, host);
      } else {
        this.sendMessageParent(data, host);
      }
    },

    sendMessageIFrames: function(data, host) {
      var iframes = document.getElementsByTagName('iframe');
      var i;

      for (i=0; i<iframes.length; i++) {
        this.sendMessage(iframes[i].contentWindow, data, host);
      }
    },

    sendMessageParent: function(data, host) {
      this.sendMessage(window.parent, data, host);
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/SCORM.js");

$require("org/mathdox/EventLib.js");
$require("org/mathdox/PostMessage.js");
$require("org/mathdox/Session.js");
$require("org/mathdox/XML.js");

$main(function() {
  org.mathdox.SCORM = $extend(org.mathdox.Session, {
    data : {},
    idstr: "MathDox Exercise System - SCORM",
    session: null,
    target : null,
    exerciseURL : null,
    loadListeners : [],

    initialize: function(debug) {
      if (debug!== undefined && debug !== null) {
        var parent = arguments.callee.parent;
        parent.initialize.call(this, debug);

        this.startDebug = function() { debug.startDebug(); };
      }

      this.target = window.parent;
      this.handleAskLMSData.obj = this;
      this.handleLMSInit.obj = this;
      this.handleSendLMSData.obj = this;
      this.lmsLoadHandler.obj = this;
      this.lmsFinish.obj = this;
    },

    addIFrameHandlers: function(iframe) {
      var eventlib = new org.mathdox.EventLib();

      eventlib.registerEventHandler(iframe, "load", this.lmsLoadHandler);
    },

    addLoadListener: function(fun) {
      this.loadListeners.push(fun);
      this.addDebug("added loadListener"); // XXX
    },

    addMessageListeners: function(messageEventLib) {
      messageEventLib.addMessageListener(this.idstr, "askLMSData", this.handleAskLMSData);
      messageEventLib.addMessageListener(this.idstr, "sendLMSInit", this.handleLMSInit);
      messageEventLib.addMessageListener(this.idstr, "sendLMSData", this.handleSendLMSData);
    },

    askLMSData: function(key) {
      var data = {
        idstr: this.idstr,
        mode: "askLMSData",
        key: key
      };
      var pm = new org.mathdox.PostMessage(this.debug);

      this.addDebug("asking for LMS Data: "+key, 2);

      pm.sendMessageParent(data, "*");
    },

    fixSlashes : function(almostXMLString) {
      return almostXMLString.split("\\/").join("/");
    },

    getData: function(key) {
      if (key === null || key === undefined ||key === "undefined") {
        return null;
      }
      if (this.data[key]!== undefined) {
        return this.data[key];
      }
    },

    getLaunchDataURL: function(data) {
      this.addDebug("function getLaunchDataURL");
      this.addDebug("DEBUG: data: "+data);
      if (data === null || data === undefined || data === "") {
        return null;
      }
      var xmlText = this.fixSlashes(data);
      var xml = new org.mathdox.XML();
      var info = xml.readXMLStringCheckEncoding(xmlText);
      var url_tags = info.getElementsByTagName("url");

      var launch_dataURL = null;

      if (url_tags !== null && url_tags.length>0 && url_tags[0] !== undefined && url_tags[0] !== null) {
        var dataURL_full=null;

        if (url_tags[0].innerHTML !== undefined && url_tags[0].innerHTML !== null) {
          dataURL_full = url_tags[0].innerHTML;
        } else if (url_tags[0].textContent !== undefined && url_tags[0].textContent !== null) { // IE or Edge
          dataURL_full = url_tags[0].textContent;
        } else if (url_tags[0].text !== undefined && url_tags[0].text !== null) { // IE 8
          dataURL_full = url_tags[0].text;
	}

        if (dataURL_full !== null) {
          var launch_dataURL_split = dataURL_full.split("/mathdox/wrapper/");
          launch_dataURL = launch_dataURL_split[launch_dataURL_split.length-1];
          this.addDebug("retrieved launch_data URL: " + launch_dataURL);
        } else {
          this.addDebug("ERROR: could not find launch_data URL");
        }
      }
      return launch_dataURL;
    },

    /* retrieve a node from previously set LMS value (info from launchdata) */
    getXMLData : function(name, node) {
      var xml = this.getData(name);
      if (xml === null || xml === undefined || xml === false) {
        return "";
      }
      if((xml.getElementsByTagName(node).length>0) && (xml.getElementsByTagName(node)[0].childNodes.length>0)) {
        return xml.getElementsByTagName(node)[0].childNodes[0].nodeValue;
      } else {
        return "";
      }
    },
    
    handleAskLMSData: function(event) {
      var pm = new org.mathdox.PostMessage(this.debug);
      var data = pm.decodeMessage(event.data);

      var source = event.source;

      var obj = arguments.callee.obj;
      obj.addDebug("handling question for LMS Data: "+data.key, 2);

      setTimeout(function() { obj.sendLMSData(data.key, source); });
    },

    handleLMSInit: function(event) {
      var pm = new org.mathdox.PostMessage(this.debug);
      var data = pm.decodeMessage(event.data);

      var obj = arguments.callee.obj;
      var debug = arguments.callee.debug;

      if (obj.startDebug !== undefined && debug!== null) {
        obj.startDebug();
      }

      setTimeout(function() { obj.lmsInit(); });
    },

    handleSendLMSData: function(event) {
      var obj = arguments.callee.obj;
      var pm = new org.mathdox.PostMessage(obj.debug);
      var data = pm.decodeMessage(event.data);
      var xml = new org.mathdox.XML();
      var value;
      
      if (data.exerciseURL !== null && data.exerciseURL !== undefined && data.exerciseURL !== "") {
        obj.addDebug("exerciseURL: " + data.exerciseURL);
	obj.exerciseURL = data.exerciseURL;
      }

      obj.addDebug("handling received LMS Data: "+data.key, 2);
      if (data.key == "cmi.launch_data") {
        /* fix slashes */
        value = obj.fixSlashes(data.value);

        obj.setData(data.key, value);

        /* also parse xml */
        obj.setData("xml_launch_data", xml.readXMLStringCheckEncoding(value)); 
      } else if (data.key == "cmi.core.lesson_status" )  {
        var oldvalue = obj.getData(data.key);

        obj.setData(data.key, data.value);

        if (data.value == "not attempted" && (oldvalue === null || oldvalue === "not attempted")) {
          obj.setData("cmi.core.lesson_status", "incomplete");

          /* send new value */
          obj.sendLMSData("cmi.core.lesson_status", null, data.exerciseURL);
        }
      } else if (data.key == "cmi.suspend_data") {
        var suspend_data = data.value;
	var uniqueid = obj.getData("exercise_uniqueid");

        /* update suspend data */
        if (suspend_data === null ||suspend_data === undefined) {
          suspend_data = "";
        } else if (suspend_data.length < 1000) {
          suspend_data = suspend_data.substring(0,1000);
        }

	// does not start with uniqueid already
	if (suspend_data.substring(0, uniqueid.length) != uniqueid ) {
          suspend_data = uniqueid + " " + suspend_data;
          obj.setData("cmi.suspend_data", suspend_data);
          obj.sendLMSData("cmi.suspend_data");
	}
      } else if (data.key == "cmi.core.score.raw") {
        // idea: check url with launch_data
        // for now: log
        obj.addDebug("xmldata: " + obj.getData("cmi.launch_data"));

        obj.setData(data.key, data.value);
      } else {
        obj.setData(data.key, data.value);
      }
    },

    lmsFinish: function() {
      var xml = new org.mathdox.XML();
      var obj = arguments.callee.obj;

      /* set session time */
      /* XXX does this do anything or is an update needed ? */
      if(obj.sessiontime !== undefined) {
        obj.setData("cmi.core.session_time",obj.sessiontime); 
      } else {
        obj.setData("cmi.core.session_time",0); 
      }

      /* create an xml document containing the LMS values */
      var xmlArr = [];

      xmlArr.push("<lms>");
      xmlArr.push("<userid>"+obj.getData("cmi.core.student_id")+"</userid>");
      xmlArr.push("<username>"+obj.getData("cmi.core.student_name")+"</username>");
      xmlArr.push("<exerciseid>"+obj.getXMLData("xml_launch_data","exercise")+"</exerciseid>");
      xmlArr.push("<status>"+obj.getData("cmi.core.lesson_status")+"</status>");
      xmlArr.push("<masteryscore>"+obj.getData("cmi.student_data.mastery_score")+"</masteryscore>");
      xmlArr.push("<totaltime>"+obj.getData("cmi.core.session_time")+"</totaltime>");

      var launchDataURL = obj.getLaunchDataURL(this.getData("cmi.launch_data"));
      if ( (obj.exerciseURL !== undefined && obj.exerciseURL !== null && launchDataURL !== null && obj.exerciseURL !== launchDataURL ) ) {
	obj.addDebug("ERROR: different URLs");
	obj.addDebug("exercise: "+obj.exerciseURL);
	obj.addDebug("LMS: "+ launchDataURL);
	obj.addDebug("Forcing score -1.");
	// force URL -1 for exercises with URL mismatch
        xmlArr.push("<score>-1</score>");
      } else {
        xmlArr.push("<score>"+obj.getData("cmi.core.score.raw")+"</score>");
      }

      xmlArr.push("<attempts>"+obj.getData("cmi.student_data.attempt_number")+"</attempts>");
      
      var uniqueid = obj.getData("exercise_uniqueid");
      if (uniqueid !== undefined && uniqueid !== null && uniqueid !== "") {
        xmlArr.push("<uniqueid>"+uniqueid+"</uniqueid>");
      }

      xmlArr.push(xml.writeXMLString(obj.getData("xml_launch_data")));
      xmlArr.push("</lms>");

      var xmlStr = xmlArr.join("");

      var url = obj.getData("mathdox.url");

      if (url !== undefined && url!== null) {
        obj.addDebug("posting "+xmlStr);
        xml.postXML(url, xmlStr);
      }
    }, 

    lmsInit: function() {
      this.addDebug("- exercise loaded ");

      this.startSessiontime();
      this.startSessionreloads();
      
      /* set session time */
      this.setData("cmi.core.session_time", 0); 

      /* set session reloads */
      this.setData("cmi.student_data.attempt_number",0); 
 
      /* get info from LMS */
      var entries = [ "cmi.core.student_id", "cmi.core.student_name",
        "cmi.core.credit", "cmi.core.lesson_status", "cmi.core.entry",
        "cmi.launch_data", "cmi.student_data.mastery_score"
      ];

      var i;
      for (i=0; i<entries.length; i++) {
        this.askLMSData(entries[i]);
      }
    },

    lmsLoad: function(iframe) {
      this.addDebug("- exercise reloaded ");
      this.lmsUpdate(iframe,true);
    },

    /* called on lmsLoad or directly when final score is known */
    lmsUpdate: function(iframe,loaded,score,exerciseURL) {
      this.addDebug("- exercise reloaded ");
      
      this.updateSessiontime();

      if (loaded) {
	this.updateSessionreloads();
      }

      /* set session time */
      if(this.sessiontime !== undefined) {
        this.setData("cmi.core.session_time",this.sessiontime); 
      } else {
        this.setData("cmi.core.session_time",0); 
      }
      /* set session reloads */
      if(this.sessionreloads !== undefined) {
        this.setData("cmi.student_data.attempt_number", this.sessionreloads); 
      } else {
        this.setData("cmi.student_data.attempt_number", 0); 
      }

      this.setURLFromIFrame(iframe);
      if (exerciseURL == undefined || exerciseURL == null) {
	exerciseURL = this.exerciseURL;
      }
      if (score === undefined || score === null) {
        this.setScoreFromIFrame(iframe, exerciseURL);
      } else {
        this.addDebug("- url "+ url);

        var maxscore=100;
        this.addDebug("- setting max score to "+ maxscore);
        this.setData("cmi.core.score.max", maxscore);
        this.sendLMSData("cmi.core.score.max", null, exerciseURL);
        this.addDebug("- setting score to "+ score);
        this.setData("cmi.core.score.raw", score);
        this.sendLMSData("cmi.core.score.raw", null, exerciseURL);
      }
      this.setUniqueidFromIFrame(iframe);

      //this.setObjectivesScoreFromIFrame(iframe);
      this.sendLMSData("cmi.core.session_time");

      if(parseInt(this.getData("cmi.core.score.raw"),10) >= parseInt(this.getData("cmi.student_data.mastery_score"),10)) {
        this.setData("cmi.core.lesson_status", "passed");
        this.sendLMSData("cmi.core.lesson_status", null, exerciseURL);
	this.sendLMSCommit();
      } else {       
	this.addDebug("the score ("+ parseInt(this.getData("cmi.core.score.raw"),10) +") is smaller than the masteryscore (" + parseInt(this.getData("cmi.student_data.mastery_score"), 10) +")",2);
        this.setData("cmi.core.lesson_status", "failed");
        this.sendLMSData("cmi.core.lesson_status", null, exerciseURL);
	this.sendLMSCommit();
      }
      
      /* call other load functions */
      var i;
      this.addDebug("calling loadListeners"); // XXX
      for (i=0; i<this.loadListeners.length;i++) {
        this.addDebug("calling loadListener "+i); // XXX
	(this.loadListeners[i])(iframe);
      }

      /* if finished call LMSFinish */
      var finished;
      if (loaded === true) {
        /* on reload, check iframe */
        finished = this.setFinishedFromIFrame(iframe);
      } else {
        /* if called without reloading, force finish */
        finished = 'true';
      }

      if (finished === 'true') {
	this.sendLMSFinish();
	this.lmsFinish();
      }
    },
    
    lmsLoadHandler: function(event) {
      arguments.callee.obj.lmsLoad(event.target);
    },

    sendLMSCommit: function(target) {
      var data = {
        idstr: this.idstr,
        mode: "sendLMSCommit"
      };
      var pm = new org.mathdox.PostMessage(this.debug);

      this.addDebug("sending LMS Commit", 2);
     
      pm.sendMessage(target, data, "*");
    },

    sendLMSFinish: function(target) {
      var data = {
        idstr: this.idstr,
        mode: "sendLMSFinish"
      };
      var pm = new org.mathdox.PostMessage(this.debug);

      this.addDebug("sending LMS Finish", 2);
     
      pm.sendMessage(target, data, "*");
    },

    sendLMSData: function(key, target, exerciseURL) {
      var data = {
        idstr: this.idstr,
        mode: "sendLMSData",
        key: key,
	exerciseURL: exerciseURL,
        value: this.getData(key)
      };

      
      var pm = new org.mathdox.PostMessage(this.debug);

      if (key == "cmi.core.session_time") {
	data.value = this.makeTime(data.value);
      }

      this.addDebug("sending LMS Data: "+key+" -> '"+data.value+"'", 2);
      this.addDebug("using exerciseURL: "+exerciseURL);

      pm.sendMessage(target, data, "*");
    },

    setData: function(key, value) {
      if (key !== null && key!== undefined) {
        this.data[key] = value;
      }
    },

    setFinishedFromIFrame: function(iframe) {
      var finished = iframe.contentDocument.getElementsByClassName("mathdox-finished");
      var result;
      if (finished.length >=1) {
        result = finished[0].textContent;;
	if (result === undefined) {
	  result = finished[0].innerText;
	}
      } else {
        result = null;
      }
      this.addDebug("- checking if finished : "+ result);
      return result;
    },

    setScoreFromIFrame: function(iframe, url) {
      var score = iframe.contentWindow.Mathdox.LMS.data.cmi_core_score_raw; 
      var maxscore=100;
      this.addDebug("- setting max score to "+ maxscore);
      this.setData("cmi.core.score.max", maxscore);
      this.sendLMSData("cmi.core.score.max", null, url);
      this.addDebug("- setting score (from IFrame) to "+ score);
      this.addDebug("- url "+ url);
      this.setData("cmi.core.score.raw", score);
      this.sendLMSData("cmi.core.score.raw", null, url);
    },

    setObjectivesScoreFromIFrame: function(iframe) {
      var scorearray = iframe.contentWindow.Mathdox.LMS.data.objectives_score; 
      // check if this can be send
      this.addDebug("- setting objectives score to " + scorearray);
      this.setData("cmi.objectives.n.score", scorearray);
      this.sendLMSData("cmi.objectives.n.score");
    },

    setUniqueidFromIFrame: function(iframe) {
      var uniqueid;
      try {
        var oldid = this.getData("exercise_uniqueid");

        uniqueid = iframe.contentWindow.Mathdox.LMS.data.exercise_uniqueid; 
        if (oldid != uniqueid) {
          this.addDebug("- setting exercise_uniqueid to "+ uniqueid);
          this.setData("exercise_uniqueid", uniqueid);
          this.askLMSData("cmi.suspend_data");
        }
      } catch(err) {
        this.addDebug("- no uniqueid found");
      }
    },

    setURLFromIFrame: function(iframe) {
      var url;
      try {
        url = iframe.contentWindow.Mathdox.LMS.data.mathdox_url; 
      } catch(err) {
        this.addDebug("- could not find mathdox_url, using default");
        url = "/mathadore/mathdox-lms-submission";
      }
      this.addDebug("- setting URL to "+ url);
      this.setData("mathdox.url", url);
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/SCORMServer.js");

$require("org/mathdox/EventLib.js");
$require("org/mathdox/FakeAPI.js");
$require("org/mathdox/PostMessage.js");
$require("org/mathdox/SCORM.js");
$require("org/mathdox/Session.js");
$require("org/mathdox/XML.js");

$main(function() {
  //var server;
  org.mathdox.SCORMServer = $extend(org.mathdox.SCORM, {
    apiHandle : null,
    apiTries : 30,
    launch_dataURL: null,

    initialize: function(debug) {
      if (debug !== undefined && debug !== null) {
        var parent = arguments.callee.parent;
        parent.initialize.call(this, debug);
      }

      this.handleAskLMSData.obj = this;
      this.handleLMSCommit.obj = this;
      this.handleLMSFinish.obj = this;
      this.handleSendLMSData.obj = this;

      var apiInitFun = function() {
        obj = arguments.callee.obj;
        obj.findAPIHandle();

        if (obj.apiHandle !== null) {
          obj.addDebug("SCORMServer: calling api.LMSInitialize");
          obj.sessionStarted = true;
          obj.apiHandle.LMSInitialize("");
        } else {
          obj.addDebug("SCORMServer: could not find APIHandle, trying again in 100 ms");
          setTimeout(arguments.callee, 100);
        }
      };

      apiInitFun.obj = this;

      apiInitFun();
    },

    addMessageListeners: function(messageEventLib) {
      var parent = arguments.callee.parent;
      parent.addMessageListeners.call(this, messageEventLib);

      messageEventLib.addMessageListener(this.idstr, "sendLMSCommit", this.handleLMSCommit);
      messageEventLib.addMessageListener(this.idstr, "sendLMSFinish", this.handleLMSFinish);
    },

    apiLMSFinish: function() {
      var retval;
      if (this.apiHandle !== null) {
        if (this.sessionStarted && ! this.sessionFinished) {
          this.sessionFinished = true;
	  this.addDebug("calling in api: LMSFinish");
          retval = this.apiHandle.LMSFinish("");
	  if (retval !== 'true') {
            var errorCode = this.apiHandle.LMSGetLastError();
	    this.addDebug("LMSFinish returned error code '"+errorCode+"'");
	  }
        }
      }
    },

    apiLMSInit: function() {
      var retval;
      if (this.apiHandle !== null) {
	this.addDebug("calling in api: LMSInitialize");
        retval = this.apiHandle.LMSInitialize("");
	if (retval !== 'true') {
          var errorCode = this.apiHandle.LMSGetLastError();
	  this.addDebug("LMSInit returned error code '"+errorCode+"'");
	}
      }
    },

    // -------------------------------------------------------------------
    // Find LMS api
    // -------------------------------------------------------------------
  
    findAPIHandle: function() {
      this.apiTries = this.apiTries -1;

      if(this.apiHandle == null) {
        this.addDebug("- findAPIHandle called to get API");
        //check this window
        this.addDebug("- look for API in window");
        this.apiHandle = this.searchAPIHandle(window);
        if ((this.apiHandle == null) && (window.opener != null) && (typeof(window.opener) != "undefined")) {
          this.addDebug("- look for API in window.opener");
          this.apiHandle = this.searchAPIHandle(window.opener);
        }
        if (this.apiHandle == null && this.apiTries <= 0) {
          this.addDebug("API not found; emulating LMS API");
          //alert('- no API Implementation found');
	  this.apiHandle = new org.mathdox.FakeAPI(this.debug);
        }
      } else {
        this.addDebug("- findAPIHandle (already found)");
      }
    },

    getData: function(key) {
      if (key === null || key === undefined) {
        return null;
      }

      var value = "";
      if (this.apiHandle !== null && this.apiHandle !== undefined) {
        value = this.apiHandle.LMSGetValue(key);
      } 

      //this.addDebug("- obtained LMS value: "+key+" -> "+value);

      return value;
    },
    
    handleSendLMSData: function(event) {
      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);
      var xml = new org.mathdox.XML();
      var value;

      var obj = arguments.callee.obj;
      obj.addDebug("handling received LMS Data: "+data.key, 2);
      if (data.exerciseURL !== undefined && data.exerciseURL!== null) {
	obj.addDebug("exerciseURL: "+data.exerciseURL);
	obj.addDebug("url: "+obj.launch_dataURL);

	obj.exerciseURL = data.exerciseURL;

	if ((obj.launch_dataURL !== null) && (obj.launch_dataURL != data.exerciseURL)) {
	  obj.addDebug("WARNING URL mismatch, not setting");
	  return;
	}
      }

      if (data.key!== null && data.key!==undefined) {
        if (data.key.indexOf('.n.') !== -1) {
	  obj.setNData(data.key, data.value);
        } else {
          obj.setData(data.key, data.value);
        }
      }
    },

    handleLMSCommit: function(event) {
      var obj = arguments.callee.obj;
      obj.addDebug("handling LMS Commit", 2);
      
      obj.sendLMSCommit("");
    },

    handleLMSFinish: function(event) {
      var obj = arguments.callee.obj;
      obj.addDebug("handling LMS Finish", 2);
      
      obj.sendLMSFinish("");
    },

    searchAPIHandle: function (win) {
      var counter = 0;
      //check win
      if(typeof win == "undefined") {
        this.addDebug("- searchAPIHandle called with undefined object");
        return null;
      }
      //search
      if (win.API_1484_11 !== null && win.API_1484_11 !== undefined) {
        this.addDebug("- found SCORM 2004 API.");
      }
      while ((win.API == null) && (win.parent != null) && (win.parent != win)) {
        counter++;
        if (counter > 7) {
           this.addDebug("- error finding API, too deeply nested.");
           return null;
        }
        win = win.parent;

        if (win.API_1484_11 !== null && win.API_1484_11 !== undefined) {
          this.addDebug("- found SCORM 2004 API.");
        }
      }
      if(win.API != null) {
        this.addDebug("- searchAPIHandle found API ("+counter+")");
        return win.API;
      } else {
        //not found
        this.addDebug("- searchAPIHandle didn't find API");
        return null;
      }
    },

    sendLMSCommit: function() {
      var retval;
      if (this.apiHandle !== null) {
        retval = this.apiHandle.LMSCommit("");
	if (retval !== 'true') {
          var errorCode = this.apiHandle.LMSGetLastError();
	  this.addDebug("LMSCommit returned error code '"+errorCode+"'");
	}
      }
    },

    sendLMSFinish: function() {
      var retval;
      if (this.apiHandle !== null) {
        retval = this.apiHandle.LMSFinish("");
	if (retval !== 'true') {
          var errorCode = this.apiHandle.LMSGetLastError();
	  this.addDebug("LMSCommit returned error code '"+errorCode+"'");
	}
      }
    },

    sendLMSData: function(key, target) {
      var data = {
        idstr: this.idstr,
        mode: "sendLMSData",
        key: key,
        value: this.getData(key)
      };

      var pm = new org.mathdox.PostMessage();

      if (key != "cmi.launch_data") {
        this.addDebug("sending LMS Data: "+key+" -> '"+data.value+"'", 2);
      } else {
	if (this.launch_dataURL === undefined || this.launch_dataURL === null) {
	  this.launch_dataURL = this.getLaunchDataURL(data.value);
	}
        this.addDebug("sending LMS Data: "+key, 2);
      }

      pm.sendMessage(target, data, "*");
    },

    sendLMSInit: function(iframe) {
      var data = {
        idstr: this.idstr,
        mode: "sendLMSInit"
      };
      var pm = new org.mathdox.PostMessage();
      
      this.addDebug("sending LMS Init", 2);
      pm.sendMessage(iframe.contentWindow, data, "*");
    },

    setData: function(key, value) {
      if (key !== null && key!== undefined && this.apiHandle !== null) {
	this.addDebug("setting LMS data: "+key+" -> "+value);
        var result = this.apiHandle.LMSSetValue(key, value);

	if (result == "false") {
          var errcode = this.apiHandle.LMSGetLastError();
	  var errorString = this.apiHandle.LMSGetErrorString(errcode);
	  this.addDebug("  error "+errcode+": "+errorString);
	}
      }
    },
    setNData: function(key, value) {
      if (key !== null && key!== undefined && this.apiHandle !== null) {
	// loop values
	var i;
	var newkey;
	var index = key.indexOf('.n.');
	var result;

        // get count (for debug)
	// not yet using for setting bound (but does everything have a _count ?)
	
	newkey = key.substring(0,index) + "._count";
	var count = this.apiHandle.LMSGetValue(newkey);
	this.addDebug("setting LMS data: " + key.substring(0,index) + " (" + count + ")");

	// skip first value
	for (i=0;i<value.length - 1;i++) {
          newkey = key.substring(0,index) + "." + i + "." + key.substring(index+3);
          this.setData("cmi.objectives."+i+".id", "objectives_test_1_"+i);
          this.setData("cmi.objectives."+i+".score.min", "0");
          this.setData("cmi.objectives."+i+".score.max", "100");
          this.setData("cmi.objectives."+i+".score.raw", value[i+1]);

	  this.addDebug("setting LMS data: " + newkey + " -> " + 0.01*parseInt(value[i+1]));
          result = this.apiHandle.LMSSetValue(newkey, value[i+1]);
	  if (result == "false") {
            var errcode = this.apiHandle.LMSGetLastError();
	    var errorString = this.apiHandle.LMSGetErrorString(errcode);
	    this.addDebug("  error "+errcode+": "+errorString);
	  }
	}
      }
    }
  });
  /*org.mathdox.SCORMServer.apiFind = function() {
    if (server !== null && server !== undefined) {
      server.findAPIHandle();
    }
    if (server !== null && server !== undefined && server.apiHandle !== null) {
      //debug.addDebug("SCORMServer: calling api.LMSInitialize");
      server.sessionStarted = true;
      server.apiHandle.LMSInitialize("");
    } else {
      setTimeout("org.mathdox.SCORMServer.apiFind()", 100);
    }
  }
  org.mathdox.SCORMServer.apiFind();*/
});
$package("org.mathdox");

$identify("org/mathdox/ScoreListener.js");

$require("org/mathdox/PostMessage.js");

$main(function() {
  org.mathdox.ScoreListener = $extend(Object, {

    scorm : null,

    idstr : "MathDox Exercise System - Exercise Score",

    exerciseURL : null,

    initialize: function(scorm, debug, exerciseURL) {
      if (scorm !== null && scorm !== undefined) {
	this.scorm = scorm;
      }
      if (debug !== null && debug !== undefined) {
	this.debug = debug;
      }
      if (exerciseURL !== null && exerciseURL !== undefined) {
	this.exerciseURL = exerciseURL;
      }
    },
    
    addDebug: function(msg) {
      if (this.debug !== null && this.debug !== undefined) {
	this.debug.addDebug(msg);
      }
    },

    addMessageListenersScorm: function(messageEventLib) {
    },
     
    addMessageListenersWrapper: function(messageEventLib) {
      this.messageListener.scoreListener = this;
      messageEventLib.addMessageListener(this.idstr, "score", this.messageListener);
    },

    messageListener: function(event) {
      var pm = new org.mathdox.PostMessage();
      var data = pm.decodeMessage(event.data);
      var scoreListener = arguments.callee.scoreListener;
      
      if (data !== undefined && data !== null) {
        if (data.idstr == scoreListener.idstr) {
	  if (data.mode == "score") {
	    this.addDebug("[score] calling LMS-update with data "+data);
	    this.addDebug("[score] using exerciseURL "+this.exerciseURL);
            scoreListener.scorm.lmsUpdate(event.target, false, data.score, this.exerciseURL);
	  }
	}
      }
    }
  });
});
$package("org.mathdox");

$identify("org/mathdox/Session.js");

$require("org/mathdox/DebugClient.js");

$main(function() {
  org.mathdox.Session = $extend(org.mathdox.DebugClient, {
    sessionTimer      : "",
    sessionStarted    : false,
    sessionFinished   : false,
    countStatus       : true,
    sessiontime       : 0,
    sessionreloads    : 0,

    initialize: function(debug) {
      if (debug !== undefined && debug!== null) {
	//if (debug.addDebug === undefined) {
        //  debug.addDebug("fout", 2);
	//}
        var parent = arguments.callee.parent;
        parent.initialize.call(this, debug);
      }
    },

    makeTime : function(seconds) {
      var time_hours;
      var time_minutes;
      var time_seconds;
      var result;

      /* make time in format HHHH:MM:SS */
      time_hours   = "" + Math.floor(seconds/3600);
      time_minutes = "" + (Math.floor(seconds/60) - (60*time_hours));
      time_seconds = "" + (seconds - (3600*time_hours) - (60*time_minutes));

      while (time_hours.length < 2) { 
        time_hours = "0" + time_hours;
      }
      while (time_minutes.length < 2) {
        time_minutes = "0" + time_minutes;
      }
      while (time_seconds.length < 2) {
        time_seconds = "0" + time_seconds;
      }

      /* return time */
      result = time_hours + ":" + time_minutes + ":" + time_seconds;

      return result;
    },

    /* update session reloads */
    startSessionreloads: function() {
      this.countStatus = true;
      this.sessionreloads = 0;
      this.addDebug("- reset Session reload counter to "+this.sessionreloads);
    },

    /* update session time (in seconds) */
    startSessiontime: function() {
      this.sessionTimer = new Date();    
      this.addDebug("- started Session time");
    },
    
    /* update session reloads */
    updateSessionreloads: function() {
      if(this.countStatus == true) {
        this.countStatus = false;
        this.sessionreloads++;
        this.addDebug("- updated Session reload counter to "+this.sessionreloads);
      } else {
        this.addDebug("- no update Session reload counter, still "+this.sessionreloads);
      }  
    },

    /* update session time (in seconds) */
    updateSessiontime: function() {
      var currentDate = new Date();    

      if(typeof this.sessionTimer.getTime != "function") {
        this.startSessiontime();
      }

      this.sessiontime = Math.ceil((currentDate.getTime() - this.sessionTimer.getTime())/1000);
      this.addDebug("- updated Session time to "+this.sessiontime);
    }
 
  });
});
$package("org.mathdox");

$identify("org/mathdox/XML.js");

$main(function() {
  org.mathdox.XML = $extend(Object, {
    postXML: function(url, xmlStr, callback) {
      // no callback, then use sendBeacon instead if possible
      if (callback === null || callback === undefined) {
        if (navigator && navigator.sendBeacon) {
          return navigator.sendBeacon(url, xmlStr);
        }
      }
     
      /* get an XMLHttpRequest object */
      var request;
      try {
        request = new XMLHttpRequest();
      } catch(exception) {
        request = new ActiveXObject("Msxml2.XMLHTTP");
      }
     
      if (callback !== undefined && callback !== null) {
        request.onreadystatechange = function () {
          // If the request is complete and was successful
          if (request.readyState === 4 && request.status === 200) {
            var type = request.getResponseHeader("Content-Type");
            if (type.indexOf("xml") !== -1 && request.responseXML) {
              callback(request.responseXML);
            } else if (type === 'application/json') {
              callback(JSON.parse(request.responseText));
            } else {
              callback(request.responseText);
            }
          }
        }
      }

      /* open synchronous request to /mathdox-lms-submission url */
      request.open("POST", url, true);

      /* send XML document containing the LMS values */
      request.setRequestHeader("Content-Type","text/xml");
 
      /* Use sendAsBinary if available (fix for FF3)  */
      if(request.sendAsBinary) {
        request.sendAsBinary(xmlStr);
      } else {
        request.send(xmlStr);
      } 
    },

    readXMLString : function(xmlStr) {
      var xmlDoc;
      var parser;
      try {
	if (window.ActiveXObject) { // NOTE: IE 11 gives false here , but supports DOMParser
          xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
          xmlDoc.async="false";
          xmlDoc.loadXML(xmlStr); 
          return xmlDoc;
	} else {
          parser=new DOMParser();
          xmlDoc=parser.parseFromString(xmlStr,"text/xml");
          return xmlDoc;
        }
      } catch(e) {
        return false;
      }
    },

    /* extra decoding */
    readXMLStringDoubleEncoded : function(xmlStr) {
      var decodedDocument = this.readXMLString("<wrapper>"+xmlStr+"</wrapper>");

      if (decodedDocument === false) {
	return false;
      }

      var decodedString = decodedDocument.documentElement.childNodes[0].nodeValue;
     
      return this.readXMLString(decodedString);
    },

    /* check if double encoding is needed */
    readXMLStringCheckEncoding: function(xmlStr) {
      if (xmlStr.indexOf("<")>-1) {
        return this.readXMLString(xmlStr);
      } else if (xmlStr.indexOf("&lt;")>-1) {
        return this.readXMLStringDoubleEncoded(xmlStr);
      } else { /* neither method can be applied, return false */
        return false;
      }
    },

    /* process xml */
    writeXMLString : function(xmlObject) {
      var xmlStr;
      try {
        xmlStr = (new XMLSerializer()).serializeToString(xmlObject);
        return xmlStr;
      } catch(e) {
        try {
          xmlStr = xmlObject.xml;
          return xmlStr;
        } catch(e2) {
          return "";
        }
      }
    }
  });
});
